/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozCC
 *
 * The Initial Developer of the Original Code is
 * Nathan R. Yergler.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Nathan R. Yergler <nathan@yergler.net>
 *
 * ***** END LICENSE BLOCK ***** */

/* mozcc.js
 * $Id: mozcc.js,v 1.13 2004/01/26 15:02:06 nathan Exp $
 */

include("chrome://mozcc/content/ccrdf.js");
include(jslib_remotefile);
include(jslib_prefs);

const pageLoaderIface = Components.interfaces.nsIWebPageDescriptor;
const nsISelectionPrivate = Components.interfaces.nsISelectionPrivate;
const nsISelectionController = Components.interfaces.nsISelectionController;
var gBrowser = null;
var gViewSourceBundle = null;
var gPrefs = null;

var gLastLineFound = '';
var gGoToLine = 0;

try {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  gPrefs = prefService.getBranch(null);
} catch (ex) {
}

var gSelectionListener = {
  timeout: 0,
  notifySelectionChanged: function(doc, sel, reason)
  {
    // Coalesce notifications within 100ms intervals.
    if (!this.timeout)
      this.timeout = setTimeout(updateStatusBar, 100);
  }
}

// addLoadListener:
//   Retrieves a handle to the browser window and connects to 
//   the load events for catching location changes

function addLoadListener() {

  // listen for navigation events
  getBrowser().addEventListener("load", onLoadContent, true);

  // listen for tab switches
  getBrowser().addEventListener("select", onLoadContent, true);

} // addLoadListener


function getButtonVisibility() {

  pref = new Prefs();
  return pref.getBool('mozcc.toolbar.hidden');

}

// extractRDF:
//   Extracts any RDF blocks from [textblock]; returns an array of strings

function extractRdf(curr_doc) {

  // check for embedded RDF
  rdf_regex = /<rdf:rdf[\w\W]*?<\/rdf:rdf>/i

  var result = new Array();
  var inputstr = curr_doc.documentElement.innerHTML;
  match = "";

  // extract any embedded RDF
  while (match != null) {
     match = inputstr.match(rdf_regex);

     if (match != null) {
        result[result.length] = match[0];
        inputstr = inputstr.substr(inputstr.search(rdf_regex) + match[0].length, inputstr.length);
     }

  } // while match is not null

  // retrieve any linked RDF
  rdfLinks = extractLinks(curr_doc); 
  for (i=0; i < rdfLinks.length; i++) {
    result[result.length] = retrieveRdf(rdfLinks[i]);
  } // for each link 

  return result;

} // extractRdf

// extractLinks
function extractLinks(current_doc) {

  var results = new Array();
  links = current_doc.documentElement.getElementsByTagName('head')[0].getElementsByTagName('link');

  for (i = 0; i < links.length; i++) {
    if (links[i].getAttribute('rel') == 'meta') {

      link_href = links[i].getAttribute('href');

      // this is an RDF link; parse it
      if (link_href.indexOf('http://') == 0) {
         // absolute link with protocol; no massaging necessary
         results.push (link_href);
      } else 
      if (link_href.charAt(0) == '/') {
         // absolute link; add the server
      } else {
         // relative link
         abs_parts = current_doc.URL.split('/');

         // assemble the new link
         abs_parts.pop()
         abs_parts.push(link_href);

         results.push(abs_parts.join('/'));

      } // relative link
    } // if the link is metadata

  }  // for each link

  return results;

}

function retrieveRdf(url) {

  var rf = new RemoteFile (url); 

  // make sure the linked file exists
  if (!(rf.exists())) return "";

  // open the file and read the contents (presumably rdf)
  rf.open();
  var contents = rf.read();

  // return the contents
  return contents

} // retrieveRdf

// onLoadContent:
//   Responds to load events; checks for RDF and updates the status bar and
//   toolbar button accordingly.

function onLoadContent() {

  var html = _content.document.documentElement.innerHTML;
  var parsed = new Array();
  var i = 0;

  // get handles to some UI elements
  button = document.getElementById('cc-button');
  statuspanel = document.getElementById('cc-status');

  // reset everything to non-RDF
  clearStatusPanel(statuspanel);
  button.disabled = true;
  button.hidden = getButtonVisibility();

  // get the RDF elements (if any)
  rdfBlocks = extractRdf(_content.document);

  // if there are any embedded RDF blocks, parse them
  uri = _content.document.URL;
  for (i=0; i < rdfBlocks.length; i++) {

    parsed.push(new ccRdf());
    parsed[parsed.length - 1].parse(rdfBlocks[i], uri);
    updateStatusBar([parsed[parsed.length - 1]]);

  } // for each embedded RDF block

  return true;

} // onLoadContent

function createImage(src, tip) {
   var newimg = document.createElement("image");

   newimg.setAttribute("src", src);
   newimg.setAttribute("alt", tip);
   newimg.tooltipText = tip;

   return newimg;
}

function clearStatusPanel(panel) {
   // remove all children from the parameter [panel]
   while (panel.hasChildNodes()) {
      panel.removeChild(panel.firstChild);
   }

 return true;
} // clearStatusPanel

function makeLicenseImages(license, container) {

         // check for public domain
         if (license.isPublicDomain()) {
            container.appendChild(
               createImage('chrome://mozcc/skin/images/pd', 'Public Domain'));
            return;
         } // if public domain

         // check for sampling licenses
         if (license.isSampling()) {
            container.appendChild(
               createImage('chrome://mozcc/skin/images/sampling', 
                           'Allows Sampling'));
         } // if is sampling
         else
         if (license.isSamplingPlus()) {
            container.appendChild(
               createImage('chrome://mozcc/skin/images/samplingplus', 
                           'Allows Sampling+'));
         } // if is sampling+

         // initialize the flag for no derivatives 
         // (since it's flagged by the lack of a permission, 
         //  not a requirement/prohibition)
         no_derivs = true;
	 notice = false;
	 attribution = false;

         // extract the properties and add them to the status bar 
         requires = license.getAll('http://web.resource.org/cc/requires');
         permits  = license.getAll('http://web.resource.org/cc/permits');
         prohibits = license.getAll('http://web.resource.org/cc/prohibits');

         for (p = 0; p < requires.length; p++) {
             // get the short target name
             objt = requires[p].Value.split('/').reverse()[0];

             switch(objt.toLowerCase()) {
               case 'notice': 
                 if (!attribution) {
                   container.appendChild(
                      createImage('chrome://mozcc/skin/images/attrib',
                                  'requires ' + objt));
                   notice=true;
                 }
                 break;

               case 'attribution': 
                 if (!notice) {
                   container.appendChild(
                      createImage('chrome://mozcc/skin/images/attrib',
                                  'requires ' + objt));
                   attribution=true;
                 }
                 break;
               case 'sharealike': 
                 container.appendChild(
                    createImage('chrome://mozcc/skin/images/share',
                                'requires ' + objt));
                 break;
             } // switch on requirement

         } // for each requirement

         for (p = 0; p < permits.length; p++) {
             // get the short target name
             objt = permits[p].Value.split('/').reverse()[0];

             switch(objt.toLowerCase()) {
                                   case 'reproduction': break;
                                   case 'distribution': break;
                                   case 'derivativeworks': 
                                        no_derivs = false;
                                        break;
             } // switch target
         } // for each permission

         for (p = 0; p < prohibits.length; p++) {
             // get the short target name
             objt = prohibits[p].Value.split('/').reverse()[0];

             switch(objt.toLowerCase()) {
                                   case 'commercialuse': 
                                        container.appendChild(
                                           createImage(
                                          'chrome://mozcc/skin/images/noncomm',
                                           'prohibits ' + objt));
                                        break;
             } // switch prohibition
         } // for each prohibition

         if (no_derivs) {
            container.appendChild(
               createImage('chrome://mozcc/skin/images/nomod', 
                            'prohibits DerivativeWorks'));
         } // if no derivatives

} // makeLicenseImages

function updateStatusBar(args) {
   // get a handle to UI elements
   statuspanel = document.getElementById('cc-status');
   button = document.getElementById('cc-button');

   works = args[0].works();

   // the work reference for this page should only have one element
   if (works.length == 1) {
       licenses = works[0].licenses();
   } else if (works.length == 0) {
       licenses = args[0].licenses();
   } else {
       return false;
   }

   // extract the license properties and update the status bar

   if (licenses.length < 1) { return false; }

      // there must be some license, so enable the button.
      button.disabled = false;

      for (l = 0; l < licenses.length; l++) {

         makeLicenseImages(licenses[l], statuspanel);

      } // for each license

   return true;

} //updateStatusBar

// showLicense:
//   displays the license information dialog for RDF contained in [aDocument];
//   derived from "view page source" code in browser.js

function showLicense(aDocument) {

  // open a dialog displaying current license information
  openDialog("chrome://mozcc/content/license.xul", "Creative Commons License",
             "chrome,resizable,modal=no,titlebar,centerscreen", 
	     aDocument);

  return true;

} // showLicense

function getBrowser()
{
  if (!gBrowser)
    gBrowser = document.getElementById("content");
  return gBrowser;
}

